# -*- coding: utf-8 -*-

###########################################################
# Finder Tags Butler
#
# Synchronize Mac OS Finder tags between several machines
#
# Copyright 2020 Borja González Seoane
#
# Contact: garaje@glezseoane.es
###########################################################

"""Properties file."""

MANIFEST_FILE_NAME = ".ftb.yaml"
MANIFEST_HEAD_COMMENT = (
    "# Finder Tags Butler manifest file\n"
    "#\n"
    "# This file is used to synchronize Mac OS Finder tags related\n"
    "# to this directory between several machines\n\n"
    "# DO NOT EDIT THIS FILE BY HAND. COULD BE CORRUPTED!\n"
    "##############################################################\n\n"
)
